@@ -4,6 +4,8 @@
 
 import asyncio
 import contextlib
+import datetime
+import json
 import sys
 import threading
 import time
@@ -36,6 +38,7 @@ from openhands.events.action import (
     ActionConfirmationStatus,
     ChangeAgentStateAction,
     CmdRunAction,
+    MCPAction,
     MessageAction,
 )
 from openhands.events.event import Event
@@ -45,8 +48,10 @@ from openhands.events.observation import (
     ErrorObservation,
     FileEditObservation,
     FileReadObservation,
+    MCPObservation,
 )
 from openhands.llm.metrics import Metrics
+from openhands.mcp.error_collector import mcp_error_collector
 
 ENABLE_STREAMING = False  # FIXME: this doesn't work
 
@@ -76,6 +81,7 @@ COMMANDS = {
     '/new': 'Create a new conversation',
     '/settings': 'Display and modify current settings',
     '/resume': 'Resume the agent when paused',
+    '/mcp': 'Manage MCP server configuration and view errors',
 }
 
 print_lock = threading.Lock()
@@ -162,6 +168,7 @@ def display_welcome_message(message: str = '') -> None:
     print_formatted_text(
         HTML("<gold>Let's start building!</gold>\n"), style=DEFAULT_STYLE
     )
+
     if message:
         print_formatted_text(
             HTML(f'{message} <grey>Type /help for help</grey>'),
@@ -186,6 +193,48 @@ def display_initial_user_prompt(prompt: str) -> None:
     )
 
 
+def display_mcp_errors() -> None:
+    """Display collected MCP errors."""
+    errors = mcp_error_collector.get_errors()
+
+    if not errors:
+        print_formatted_text(HTML('<ansigreen>✓ No MCP errors detected</ansigreen>\n'))
+        return
+
+    print_formatted_text(
+        HTML(
+            f'<ansired>✗ {len(errors)} MCP error(s) detected during startup:</ansired>\n'
+        )
+    )
+
+    for i, error in enumerate(errors, 1):
+        # Format timestamp
+        timestamp = datetime.datetime.fromtimestamp(error.timestamp).strftime(
+            '%H:%M:%S'
+        )
+
+        # Create error display text
+        error_text = (
+            f'[{timestamp}] {error.server_type.upper()} Server: {error.server_name}\n'
+        )
+        error_text += f'Error: {error.error_message}\n'
+        if error.exception_details:
+            error_text += f'Details: {error.exception_details}'
+
+        container = Frame(
+            TextArea(
+                text=error_text,
+                read_only=True,
+                style='ansired',
+                wrap_lines=True,
+            ),
+            title=f'MCP Error #{i}',
+            style='ansired',
+        )
+        print_container(container)
+        print_formatted_text('')  # Add spacing between errors
+
+
 # Prompt output display functions
 def display_thought_if_new(thought: str) -> None:
     """Display a thought only if it hasn't been displayed recently."""
@@ -215,6 +264,8 @@ def display_event(event: Event, config: OpenHandsConfig) -> None:
 
             if event.confirmation_state == ActionConfirmationStatus.CONFIRMED:
                 initialize_streaming_output()
+        elif isinstance(event, MCPAction):
+            display_mcp_action(event)
         elif isinstance(event, Action):
             # For other actions, display thoughts normally
             if hasattr(event, 'thought') and event.thought:
@@ -232,6 +283,8 @@ def display_event(event: Event, config: OpenHandsConfig) -> None:
             display_file_edit(event)
         elif isinstance(event, FileReadObservation):
             display_file_read(event)
+        elif isinstance(event, MCPObservation):
+            display_mcp_observation(event)
         elif isinstance(event, AgentStateChangedObservation):
             display_agent_state_change_message(event.agent_state)
         elif isinstance(event, ErrorObservation):
@@ -337,6 +390,66 @@ def display_file_read(event: FileReadObservation) -> None:
     print_container(container)
 
 
+def display_mcp_action(event: MCPAction) -> None:
+    """Display an MCP action in the CLI."""
+    # Format the arguments for display
+    args_text = ''
+    if event.arguments:
+        try:
+            args_text = json.dumps(event.arguments, indent=2)
+        except (TypeError, ValueError):
+            args_text = str(event.arguments)
+
+    # Create the display text
+    display_text = f'Tool: {event.name}'
+    if args_text:
+        display_text += f'\n\nArguments:\n{args_text}'
+
+    container = Frame(
+        TextArea(
+            text=display_text,
+            read_only=True,
+            style='ansiblue',
+            wrap_lines=True,
+        ),
+        title='MCP Tool Call',
+        style='ansiblue',
+    )
+    print_formatted_text('')
+    print_container(container)
+
+
+def display_mcp_observation(event: MCPObservation) -> None:
+    """Display an MCP observation in the CLI."""
+    # Format the content for display
+    content = event.content.strip() if event.content else 'No output'
+
+    # Add tool name and arguments info if available
+    display_text = content
+    if event.name:
+        header = f'Tool: {event.name}'
+        if event.arguments:
+            try:
+                args_text = json.dumps(event.arguments, indent=2)
+                header += f'\nArguments: {args_text}'
+            except (TypeError, ValueError):
+                header += f'\nArguments: {event.arguments}'
+        display_text = f'{header}\n\nResult:\n{content}'
+
+    container = Frame(
+        TextArea(
+            text=display_text,
+            read_only=True,
+            style=COLOR_GREY,
+            wrap_lines=True,
+        ),
+        title='MCP Tool Result',
+        style=f'fg:{COLOR_GREY}',
+    )
+    print_formatted_text('')
+    print_container(container)
+
+
 def initialize_streaming_output():
     """Initialize the streaming output TextArea."""
     if not ENABLE_STREAMING:
@@ -589,34 +702,19 @@ async def read_prompt_input(
 
 async def read_confirmation_input(config: OpenHandsConfig) -> str:
     try:
-        prompt_session = create_prompt_session(config)
+        choices = [
+            'Yes, proceed',
+            'No (and allow to enter instructions)',
+            "Always proceed (don't ask again)",
+        ]
 
-        while True:
-            with patch_stdout():
-                print_formatted_text('')
-                confirmation: str = await prompt_session.prompt_async(
-                    HTML('<gold>Proceed with action? (y)es/(n)o/(a)lways > </gold>'),
-                )
+        # keep the outer coroutine responsive by using asyncio.to_thread which puts the blocking call app.run() of cli_confirm() in a separate thread
+        index = await asyncio.to_thread(
+            cli_confirm, config, 'Choose an option:', choices
+        )
 
-                confirmation = (
-                    '' if confirmation is None else confirmation.strip().lower()
-                )
+        return {0: 'yes', 1: 'no', 2: 'always'}.get(index, 'no')
 
-                if confirmation in ['y', 'yes']:
-                    return 'yes'
-                elif confirmation in ['n', 'no']:
-                    return 'no'
-                elif confirmation in ['a', 'always']:
-                    return 'always'
-                else:
-                    # Display error message for invalid input
-                    print_formatted_text('')
-                    print_formatted_text(
-                        HTML(
-                            '<ansired>Invalid input. Please enter (y)es, (n)o, or (a)lways.</ansired>'
-                        )
-                    )
-                    # Continue the loop to re-prompt
     except (KeyboardInterrupt, EOFError):
         return 'no'
 
