@@ -1,37 +1,47 @@
 from openhands.core.config.agent_config import AgentConfig
-from openhands.core.config.app_config import AppConfig
+from openhands.core.config.cli_config import CLIConfig
 from openhands.core.config.config_utils import (
     OH_DEFAULT_AGENT,
     OH_MAX_ITERATIONS,
     get_field_info,
 )
+from openhands.core.config.extended_config import ExtendedConfig
 from openhands.core.config.llm_config import LLMConfig
+from openhands.core.config.mcp_config import MCPConfig
+from openhands.core.config.openhands_config import OpenHandsConfig
 from openhands.core.config.sandbox_config import SandboxConfig
 from openhands.core.config.security_config import SecurityConfig
 from openhands.core.config.utils import (
     finalize_config,
+    get_agent_config_arg,
     get_llm_config_arg,
     get_parser,
-    load_app_config,
     load_from_env,
     load_from_toml,
+    load_openhands_config,
     parse_arguments,
+    setup_config_from_args,
 )
 
 __all__ = [
     'OH_DEFAULT_AGENT',
     'OH_MAX_ITERATIONS',
     'AgentConfig',
-    'AppConfig',
+    'CLIConfig',
+    'OpenHandsConfig',
+    'MCPConfig',
     'LLMConfig',
     'SandboxConfig',
     'SecurityConfig',
-    'load_app_config',
+    'ExtendedConfig',
+    'load_openhands_config',
     'load_from_env',
     'load_from_toml',
     'finalize_config',
+    'get_agent_config_arg',
     'get_llm_config_arg',
     'get_field_info',
     'get_parser',
     'parse_arguments',
+    'setup_config_from_args',
 ]
