@@ -5,7 +5,7 @@ import platform
 import sys
 from ast import literal_eval
 from types import UnionType
-from typing import MutableMapping, get_args, get_origin
+from typing import MutableMapping, get_args, get_origin, get_type_hints
 from uuid import uuid4
 
 import toml
@@ -154,8 +154,22 @@ def load_from_toml(cfg: OpenHandsConfig, toml_file: str = 'config.toml') -> None
         core_config = toml_config['core']
 
     # Process core section if present
+    cfg_type_hints = get_type_hints(cfg.__class__)
     for key, value in core_config.items():
         if hasattr(cfg, key):
+            # Get expected type of the attribute
+            expected_type = cfg_type_hints.get(key, None)
+
+            # Check if expected_type is a Union that includes SecretStr and value is str, e.g. search_api_key
+            if expected_type:
+                origin = get_origin(expected_type)
+                args = get_args(expected_type)
+
+                if origin is UnionType and SecretStr in args and isinstance(value, str):
+                    value = SecretStr(value)
+                elif expected_type is SecretStr and isinstance(value, str):
+                    value = SecretStr(value)
+
             setattr(cfg, key, value)
         else:
             logger.openhands_logger.warning(
@@ -323,7 +337,7 @@ def finalize_config(cfg: OpenHandsConfig) -> None:
     if cfg.workspace_base is not None or cfg.workspace_mount_path is not None:
         logger.openhands_logger.warning(
             'DEPRECATED: The WORKSPACE_BASE and WORKSPACE_MOUNT_PATH environment variables are deprecated. '
-            "Please use RUNTIME_MOUNT instead, e.g. 'RUNTIME_MOUNT=/my/host/dir:/workspace:rw'"
+            "Please use SANDBOX_VOLUMES instead, e.g. 'SANDBOX_VOLUMES=/my/host/dir:/workspace:rw'"
         )
     if cfg.sandbox.volumes is not None:
         # Split by commas to handle multiple mounts
@@ -501,7 +515,14 @@ def get_llm_config_arg(
     if llm_config_arg.startswith('llm.'):
         llm_config_arg = llm_config_arg[4:]
 
-    logger.openhands_logger.debug(f'Loading llm config from {llm_config_arg}')
+    logger.openhands_logger.debug(
+        f'Loading llm config "{llm_config_arg}" from {toml_file}'
+    )
+
+    # Check if the file exists
+    if not os.path.exists(toml_file):
+        logger.openhands_logger.debug(f'Config file not found: {toml_file}')
+        return None
 
     # load the toml file
     try:
@@ -519,7 +540,10 @@ def get_llm_config_arg(
     # update the llm config with the specified section
     if 'llm' in toml_config and llm_config_arg in toml_config['llm']:
         return LLMConfig(**toml_config['llm'][llm_config_arg])
-    logger.openhands_logger.debug(f'Loading from toml failed for {llm_config_arg}')
+
+    logger.openhands_logger.debug(
+        f'LLM config "{llm_config_arg}" not found in {toml_file}'
+    )
     return None
 
 
@@ -827,20 +851,52 @@ def setup_config_from_args(args: argparse.Namespace) -> OpenHandsConfig:
     """Load config from toml and override with command line arguments.
 
     Common setup used by both CLI and main.py entry points.
+
+    Configuration precedence (from highest to lowest):
+    1. CLI parameters (e.g., -l for LLM config)
+    2. config.toml in current directory (or --config-file location if specified)
+    3. ~/.openhands/settings.json and ~/.openhands/config.toml
     """
     # Load base config from toml and env vars
     config = load_openhands_config(config_file=args.config_file)
 
     # Override with command line arguments if provided
     if args.llm_config:
-        # if we didn't already load it, get it from the toml file
+        logger.openhands_logger.debug(f'CLI specified LLM config: {args.llm_config}')
+
+        # Check if the LLM config is NOT in the loaded configs
         if args.llm_config not in config.llms:
-            llm_config = get_llm_config_arg(args.llm_config)
+            # Try to load from the specified config file
+            llm_config = get_llm_config_arg(args.llm_config, args.config_file)
+
+            # If not found in the specified config file, try the user's config.toml
+            if llm_config is None and args.config_file != os.path.join(
+                os.path.expanduser('~'), '.openhands', 'config.toml'
+            ):
+                user_config = os.path.join(
+                    os.path.expanduser('~'), '.openhands', 'config.toml'
+                )
+                if os.path.exists(user_config):
+                    logger.openhands_logger.debug(
+                        f"Trying to load LLM config '{args.llm_config}' from user config: {user_config}"
+                    )
+                    llm_config = get_llm_config_arg(args.llm_config, user_config)
         else:
+            # If it's already in the loaded configs, use that
             llm_config = config.llms[args.llm_config]
+            logger.openhands_logger.debug(
+                f"Using LLM config '{args.llm_config}' from loaded configuration"
+            )
         if llm_config is None:
-            raise ValueError(f'Invalid toml file, cannot read {args.llm_config}')
+            raise ValueError(
+                f"Cannot find LLM configuration '{args.llm_config}' in any config file"
+            )
+
+        # Set this as the default LLM config (highest precedence)
         config.set_llm_config(llm_config)
+        logger.openhands_logger.debug(
+            f'Set LLM config from CLI parameter: {args.llm_config}'
+        )
 
     # Override default agent if provided
     if args.agent_cls:
