@@ -6,12 +6,39 @@ import sys
 import traceback
 from datetime import datetime
 from types import TracebackType
-from typing import Any, Literal, Mapping
+from typing import Any, Literal, Mapping, MutableMapping, TextIO
 
+import litellm
+from pythonjsonlogger.json import JsonFormatter
 from termcolor import colored
 
 LOG_LEVEL = os.getenv('LOG_LEVEL', 'INFO').upper()
 DEBUG = os.getenv('DEBUG', 'False').lower() in ['true', '1', 'yes']
+DEBUG_LLM = os.getenv('DEBUG_LLM', 'False').lower() in ['true', '1', 'yes']
+
+# Structured logs with JSON, disabled by default
+LOG_JSON = os.getenv('LOG_JSON', 'False').lower() in ['true', '1', 'yes']
+LOG_JSON_LEVEL_KEY = os.getenv('LOG_JSON_LEVEL_KEY', 'level')
+
+
+# Configure litellm logging based on DEBUG_LLM
+if DEBUG_LLM:
+    confirmation = input(
+        '\n⚠️ WARNING: You are enabling DEBUG_LLM which may expose sensitive information like API keys.\n'
+        'This should NEVER be enabled in production.\n'
+        "Type 'y' to confirm you understand the risks: "
+    )
+    if confirmation.lower() == 'y':
+        litellm.suppress_debug_info = False
+        litellm.set_verbose = True
+    else:
+        print('DEBUG_LLM disabled due to lack of confirmation')
+        litellm.suppress_debug_info = True
+        litellm.set_verbose = False
+else:
+    litellm.suppress_debug_info = True
+    litellm.set_verbose = False
+
 if DEBUG:
     LOG_LEVEL = 'DEBUG'
 
@@ -20,6 +47,9 @@ DISABLE_COLOR_PRINTING = False
 
 LOG_ALL_EVENTS = os.getenv('LOG_ALL_EVENTS', 'False').lower() in ['true', '1', 'yes']
 
+# Controls whether to stream Docker container logs
+DEBUG_RUNTIME = os.getenv('DEBUG_RUNTIME', 'False').lower() in ['true', '1', 'yes']
+
 ColorType = Literal[
     'red',
     'green',
@@ -49,12 +79,30 @@ LOG_COLORS: Mapping[str, ColorType] = {
 }
 
 
+class StackInfoFilter(logging.Filter):
+    def filter(self, record: logging.LogRecord) -> bool:
+        if record.levelno >= logging.ERROR:
+            # Only add stack trace info if there's an actual exception
+            exc_info = sys.exc_info()
+            if exc_info and exc_info[0] is not None:
+                # Capture the current stack trace as a string
+                stack = traceback.format_stack()
+                # Remove the last entries which are related to the logging machinery
+                stack = stack[:-3]  # Adjust this number if needed
+                # Join the stack frames into a single string
+                stack_str = ''.join(stack)
+                setattr(record, 'stack_info', stack_str)
+                setattr(record, 'exc_info', exc_info)
+        return True
+
+
 class NoColorFormatter(logging.Formatter):
     """Formatter for non-colored logging in files."""
 
     def format(self, record: logging.LogRecord) -> str:
         # Create a deep copy of the record to avoid modifying the original
-        new_record: logging.LogRecord = copy.deepcopy(record)
+        new_record = _fix_record(record)
+
         # Strip ANSI color codes from the message
         new_record.msg = strip_ansi(new_record.msg)
 
@@ -74,9 +122,9 @@ def strip_ansi(s: str) -> str:
 
 
 class ColoredFormatter(logging.Formatter):
-    def format(self, record):
-        msg_type = record.__dict__.get('msg_type')
-        event_source = record.__dict__.get('event_source')
+    def format(self, record: logging.LogRecord) -> str:
+        msg_type = record.__dict__.get('msg_type', '')
+        event_source = record.__dict__.get('event_source', '')
         if event_source:
             new_msg_type = f'{event_source.upper()}_{msg_type}'
             if new_msg_type in LOG_COLORS:
@@ -98,7 +146,19 @@ class ColoredFormatter(logging.Formatter):
                 return f'{msg}'
             else:
                 return record.msg
-        return super().format(record)
+
+        new_record = _fix_record(record)
+        return super().format(new_record)
+
+
+def _fix_record(record: logging.LogRecord) -> logging.LogRecord:
+    new_record = copy.copy(record)
+    # The formatter expects non boolean values, and will raise an exception if there is a boolean - so we fix these
+    # LogRecord attributes are dynamically typed
+    if getattr(new_record, 'exc_info', None) is True:
+        setattr(new_record, 'exc_info', sys.exc_info())
+        setattr(new_record, 'stack_info', None)
+    return new_record
 
 
 file_formatter = NoColorFormatter(
@@ -112,31 +172,34 @@ class RollingLogger:
     max_lines: int
     char_limit: int
     log_lines: list[str]
+    all_lines: str
 
-    def __init__(self, max_lines=10, char_limit=80):
+    def __init__(self, max_lines: int = 10, char_limit: int = 80) -> None:
         self.max_lines = max_lines
         self.char_limit = char_limit
         self.log_lines = [''] * self.max_lines
+        self.all_lines = ''
 
-    def is_enabled(self):
+    def is_enabled(self) -> bool:
         return DEBUG and sys.stdout.isatty()
 
-    def start(self, message=''):
+    def start(self, message: str = '') -> None:
         if message:
             print(message)
         self._write('\n' * self.max_lines)
         self._flush()
 
-    def add_line(self, line):
+    def add_line(self, line: str) -> None:
         self.log_lines.pop(0)
         self.log_lines.append(line[: self.char_limit])
         self.print_lines()
+        self.all_lines += line + '\n'
 
-    def write_immediately(self, line):
+    def write_immediately(self, line: str) -> None:
         self._write(line)
         self._flush()
 
-    def print_lines(self):
+    def print_lines(self) -> None:
         """Display the last n log_lines in the console (not for file logging).
 
         This will create the effect of a rolling display in the console.
@@ -145,32 +208,48 @@ class RollingLogger:
         for line in self.log_lines:
             self.replace_current_line(line)
 
-    def move_back(self, amount=-1):
+    def move_back(self, amount: int = -1) -> None:
         r"""'\033[F' moves the cursor up one line."""
         if amount == -1:
             amount = self.max_lines
         self._write('\033[F' * (self.max_lines))
         self._flush()
 
-    def replace_current_line(self, line=''):
+    def replace_current_line(self, line: str = '') -> None:
         r"""'\033[2K\r' clears the line and moves the cursor to the beginning of the line."""
         self._write('\033[2K' + line + '\n')
         self._flush()
 
-    def _write(self, line):
+    def _write(self, line: str) -> None:
         if not self.is_enabled():
             return
         sys.stdout.write(line)
 
-    def _flush(self):
+    def _flush(self) -> None:
         if not self.is_enabled():
             return
         sys.stdout.flush()
 
 
 class SensitiveDataFilter(logging.Filter):
-    def filter(self, record):
-        # start with attributes
+    def filter(self, record: logging.LogRecord) -> bool:
+        # Gather sensitive values which should not ever appear in the logs.
+        sensitive_values = []
+        for key, value in os.environ.items():
+            key_upper = key.upper()
+            if (
+                len(value) > 2
+                and value != 'default'
+                and any(s in key_upper for s in ('SECRET', '_KEY', '_CODE', '_TOKEN'))
+            ):
+                sensitive_values.append(value)
+
+        # Replace sensitive values from env!
+        msg = record.getMessage()
+        for sensitive_value in sensitive_values:
+            msg = msg.replace(sensitive_value, '******')
+
+        # Replace obvious sensitive values from log itself...
         sensitive_patterns = [
             'api_key',
             'aws_access_key_id',
@@ -180,53 +259,75 @@ class SensitiveDataFilter(logging.Filter):
             'jwt_secret',
             'modal_api_token_id',
             'modal_api_token_secret',
+            'llm_api_key',
+            'sandbox_env_github_token',
+            'runloop_api_key',
+            'daytona_api_key',
         ]
 
         # add env var names
         env_vars = [attr.upper() for attr in sensitive_patterns]
         sensitive_patterns.extend(env_vars)
 
-        # and some special cases
-        sensitive_patterns.append('JWT_SECRET')
-        sensitive_patterns.append('LLM_API_KEY')
-        sensitive_patterns.append('GITHUB_TOKEN')
-        sensitive_patterns.append('SANDBOX_ENV_GITHUB_TOKEN')
-
-        # this also formats the message with % args
-        msg = record.getMessage()
-        record.args = ()
-
         for attr in sensitive_patterns:
             pattern = rf"{attr}='?([\w-]+)'?"
             msg = re.sub(pattern, f"{attr}='******'", msg)
 
-        # passed with msg
+        # Update the record
         record.msg = msg
+        record.args = ()
+
         return True
 
 
-def get_console_handler(log_level: int = logging.INFO, extra_info: str | None = None):
+def get_console_handler(log_level: int = logging.INFO) -> logging.StreamHandler:
     """Returns a console handler for logging."""
     console_handler = logging.StreamHandler()
     console_handler.setLevel(log_level)
     formatter_str = '\033[92m%(asctime)s - %(name)s:%(levelname)s\033[0m: %(filename)s:%(lineno)s - %(message)s'
-    if extra_info:
-        formatter_str = f'{extra_info} - ' + formatter_str
     console_handler.setFormatter(ColoredFormatter(formatter_str, datefmt='%H:%M:%S'))
     return console_handler
 
 
-def get_file_handler(log_dir: str, log_level: int = logging.INFO):
+def get_file_handler(
+    log_dir: str, log_level: int = logging.INFO
+) -> logging.FileHandler:
     """Returns a file handler for logging."""
     os.makedirs(log_dir, exist_ok=True)
     timestamp = datetime.now().strftime('%Y-%m-%d')
     file_name = f'openhands_{timestamp}.log'
     file_handler = logging.FileHandler(os.path.join(log_dir, file_name))
     file_handler.setLevel(log_level)
-    file_handler.setFormatter(file_formatter)
+    if LOG_JSON:
+        file_handler.setFormatter(json_formatter())
+    else:
+        file_handler.setFormatter(file_formatter)
     return file_handler
 
 
+def json_formatter() -> JsonFormatter:
+    return JsonFormatter(
+        '{message}{levelname}',
+        style='{',
+        rename_fields={'levelname': LOG_JSON_LEVEL_KEY},
+        timestamp=True,
+    )
+
+
+def json_log_handler(
+    level: int = logging.INFO,
+    _out: TextIO = sys.stdout,
+) -> logging.Handler:
+    """
+    Configure logger instance for structured logging as json lines.
+    """
+
+    handler = logging.StreamHandler(_out)
+    handler.setLevel(level)
+    handler.setFormatter(json_formatter())
+    return handler
+
+
 # Set up logging
 logging.basicConfig(level=logging.ERROR)
 
@@ -257,11 +358,18 @@ if LOG_LEVEL in logging.getLevelNamesMapping():
     current_log_level = logging.getLevelNamesMapping()[LOG_LEVEL]
 openhands_logger.setLevel(current_log_level)
 
+if DEBUG:
+    openhands_logger.addFilter(StackInfoFilter())
+
 if current_log_level == logging.DEBUG:
     LOG_TO_FILE = True
     openhands_logger.debug('DEBUG mode enabled.')
 
-openhands_logger.addHandler(get_console_handler(current_log_level))
+if LOG_JSON:
+    openhands_logger.addHandler(json_log_handler(current_log_level))
+else:
+    openhands_logger.addHandler(get_console_handler(current_log_level))
+
 openhands_logger.addFilter(SensitiveDataFilter(openhands_logger.name))
 openhands_logger.propagate = False
 openhands_logger.debug('Logging initialized')
@@ -278,16 +386,34 @@ if LOG_TO_FILE:
     )  # default log to project root
     openhands_logger.debug(f'Logging to file in: {LOG_DIR}')
 
-# Exclude LiteLLM from logging output
+# Exclude LiteLLM from logging output as it can leak keys
 logging.getLogger('LiteLLM').disabled = True
 logging.getLogger('LiteLLM Router').disabled = True
 logging.getLogger('LiteLLM Proxy').disabled = True
 
+# Exclude loquacious loggers
+LOQUACIOUS_LOGGERS = [
+    'engineio',
+    'engineio.server',
+    'socketio',
+    'socketio.client',
+    'socketio.server',
+]
+
+for logger_name in LOQUACIOUS_LOGGERS:
+    logging.getLogger(logger_name).setLevel('WARNING')
+
 
 class LlmFileHandler(logging.FileHandler):
     """LLM prompt and response logging."""
 
-    def __init__(self, filename, mode='a', encoding='utf-8', delay=False):
+    def __init__(
+        self,
+        filename: str,
+        mode: str = 'a',
+        encoding: str = 'utf-8',
+        delay: bool = False,
+    ) -> None:
         """Initializes an instance of LlmFileHandler.
 
         Args:
@@ -318,7 +444,7 @@ class LlmFileHandler(logging.FileHandler):
         self.baseFilename = os.path.join(self.log_directory, filename)
         super().__init__(self.baseFilename, mode, encoding, delay)
 
-    def emit(self, record):
+    def emit(self, record: logging.LogRecord) -> None:
         """Emits a log record.
 
         Args:
@@ -333,7 +459,7 @@ class LlmFileHandler(logging.FileHandler):
         self.message_counter += 1
 
 
-def _get_llm_file_handler(name: str, log_level: int):
+def _get_llm_file_handler(name: str, log_level: int) -> LlmFileHandler:
     # The 'delay' parameter, when set to True, postpones the opening of the log file
     # until the first log message is emitted.
     llm_file_handler = LlmFileHandler(name, delay=True)
@@ -342,7 +468,7 @@ def _get_llm_file_handler(name: str, log_level: int):
     return llm_file_handler
 
 
-def _setup_llm_logger(name: str, log_level: int):
+def _setup_llm_logger(name: str, log_level: int) -> logging.Logger:
     logger = logging.getLogger(name)
     logger.propagate = False
     logger.setLevel(log_level)
@@ -353,3 +479,26 @@ def _setup_llm_logger(name: str, log_level: int):
 
 llm_prompt_logger = _setup_llm_logger('prompt', current_log_level)
 llm_response_logger = _setup_llm_logger('response', current_log_level)
+
+
+class OpenHandsLoggerAdapter(logging.LoggerAdapter):
+    extra: dict
+
+    def __init__(
+        self, logger: logging.Logger = openhands_logger, extra: dict | None = None
+    ) -> None:
+        self.logger = logger
+        self.extra = extra or {}
+
+    def process(
+        self, msg: str, kwargs: MutableMapping[str, Any]
+    ) -> tuple[str, MutableMapping[str, Any]]:
+        """
+        If 'extra' is supplied in kwargs, merge it with the adapters 'extra' dict
+        Starting in Python 3.13, LoggerAdapter's merge_extra option will do this.
+        """
+        if 'extra' in kwargs and isinstance(kwargs['extra'], dict):
+            kwargs['extra'] = {**self.extra, **kwargs['extra']}
+        else:
+            kwargs['extra'] = self.extra
+        return msg, kwargs
