@@ -3,8 +3,9 @@ from openhands.events.action.agent import (
     AgentDelegateAction,
     AgentFinishAction,
     AgentRejectAction,
-    AgentSummarizeAction,
+    AgentThinkAction,
     ChangeAgentStateAction,
+    RecallAction,
 )
 from openhands.events.action.browse import BrowseInteractiveAction, BrowseURLAction
 from openhands.events.action.commands import CmdRunAction, IPythonRunCellAction
@@ -14,8 +15,8 @@ from openhands.events.action.files import (
     FileReadAction,
     FileWriteAction,
 )
-from openhands.events.action.message import MessageAction
-from openhands.events.action.tasks import AddTaskAction, ModifyTaskAction
+from openhands.events.action.mcp import MCPAction
+from openhands.events.action.message import MessageAction, SystemMessageAction
 
 __all__ = [
     'Action',
@@ -29,11 +30,12 @@ __all__ = [
     'AgentFinishAction',
     'AgentRejectAction',
     'AgentDelegateAction',
-    'AgentSummarizeAction',
-    'AddTaskAction',
-    'ModifyTaskAction',
     'ChangeAgentStateAction',
     'IPythonRunCellAction',
     'MessageAction',
+    'SystemMessageAction',
     'ActionConfirmationStatus',
+    'AgentThinkAction',
+    'RecallAction',
+    'MCPAction',
 ]
