@@ -1,10 +1,16 @@
+from typing import Any
+
 from openhands.core.exceptions import LLMMalformedActionError
 from openhands.events.action.action import Action
 from openhands.events.action.agent import (
     AgentDelegateAction,
     AgentFinishAction,
     AgentRejectAction,
+    AgentThinkAction,
     ChangeAgentStateAction,
+    CondensationAction,
+    CondensationRequestAction,
+    RecallAction,
 )
 from openhands.events.action.browse import BrowseInteractiveAction, BrowseURLAction
 from openhands.events.action.commands import (
@@ -17,8 +23,8 @@ from openhands.events.action.files import (
     FileReadAction,
     FileWriteAction,
 )
-from openhands.events.action.message import MessageAction
-from openhands.events.action.tasks import AddTaskAction, ModifyTaskAction
+from openhands.events.action.mcp import MCPAction
+from openhands.events.action.message import MessageAction, SystemMessageAction
 
 actions = (
     NullAction,
@@ -29,18 +35,60 @@ actions = (
     FileReadAction,
     FileWriteAction,
     FileEditAction,
+    AgentThinkAction,
     AgentFinishAction,
     AgentRejectAction,
     AgentDelegateAction,
-    AddTaskAction,
-    ModifyTaskAction,
+    RecallAction,
     ChangeAgentStateAction,
     MessageAction,
+    SystemMessageAction,
+    CondensationAction,
+    CondensationRequestAction,
+    MCPAction,
 )
 
 ACTION_TYPE_TO_CLASS = {action_class.action: action_class for action_class in actions}  # type: ignore[attr-defined]
 
 
+def handle_action_deprecated_args(args: dict[str, Any]) -> dict[str, Any]:
+    # keep_prompt has been deprecated in https://github.com/All-Hands-AI/OpenHands/pull/4881
+    if 'keep_prompt' in args:
+        args.pop('keep_prompt')
+
+    # Handle translated_ipython_code deprecation
+    if 'translated_ipython_code' in args:
+        code = args.pop('translated_ipython_code')
+
+        # Check if it's a file_editor call using a prefix check for efficiency
+        file_editor_prefix = 'print(file_editor(**'
+        if (
+            code is not None
+            and code.startswith(file_editor_prefix)
+            and code.endswith('))')
+        ):
+            try:
+                # Extract and evaluate the dictionary string
+                import ast
+
+                # Extract the dictionary string between the prefix and the closing parentheses
+                dict_str = code[len(file_editor_prefix) : -2]  # Remove prefix and '))'
+                file_args = ast.literal_eval(dict_str)
+
+                # Update args with the extracted file editor arguments
+                args.update(file_args)
+            except (ValueError, SyntaxError):
+                # If parsing fails, just remove the translated_ipython_code
+                pass
+
+        if args.get('command') == 'view':
+            args.pop(
+                'command'
+            )  # "view" will be translated to FileReadAction which doesn't have a command argument
+
+    return args
+
+
 def action_from_dict(action: dict) -> Action:
     if not isinstance(action, dict):
         raise LLMMalformedActionError('action must be a dictionary')
@@ -70,10 +118,14 @@ def action_from_dict(action: dict) -> Action:
     if 'images_urls' in args:
         args['image_urls'] = args.pop('images_urls')
 
+    # handle deprecated args
+    args = handle_action_deprecated_args(args)
+
     try:
         decoded_action = action_class(**args)
         if 'timeout' in action:
-            decoded_action.timeout = action['timeout']
+            blocking = args.get('blocking', False)
+            decoded_action.set_hard_timeout(action['timeout'], blocking=blocking)
 
         # Set timestamp if it was provided
         if timestamp:
@@ -83,4 +135,5 @@ def action_from_dict(action: dict) -> Action:
         raise LLMMalformedActionError(
             f'action={action} has the wrong arguments: {str(e)}'
         )
+    assert isinstance(decoded_action, Action)
     return decoded_action
