@@ -17,7 +17,9 @@ import httpx
 
 from openhands.core.config import OpenHandsConfig, SandboxConfig
 from openhands.core.config.mcp_config import MCPConfig, MCPStdioServerConfig
-from openhands.core.exceptions import AgentRuntimeDisconnectedError
+from openhands.core.exceptions import (
+    AgentRuntimeDisconnectedError,
+)
 from openhands.core.logger import openhands_logger as logger
 from openhands.events import EventSource, EventStream, EventStreamSubscriber
 from openhands.events.action import (
@@ -131,7 +133,8 @@ class Runtime(FileEditRuntimeMixin):
         git_provider_tokens: PROVIDER_TOKEN_TYPE | None = None,
     ):
         self.git_handler = GitHandler(
-            execute_shell_fn=self._execute_shell_fn_git_handler
+            execute_shell_fn=self._execute_shell_fn_git_handler,
+            create_file_fn=self._create_file_fn_git_handler,
         )
         self.sid = sid
         self.event_stream = event_stream
@@ -340,14 +343,23 @@ class Runtime(FileEditRuntimeMixin):
                 observation: Observation = await self.call_tool_mcp(event)
             else:
                 observation = await call_sync_from_async(self.run_action, event)
-        except Exception as e:
-            runtime_status = RuntimeStatus.ERROR
-            if isinstance(e, (httpx.NetworkError, AgentRuntimeDisconnectedError)):
-                runtime_status = RuntimeStatus.ERROR_RUNTIME_DISCONNECTED
+        except PermissionError as e:
+            # Handle PermissionError specially - convert to ErrorObservation
+            # so the agent can receive feedback and continue execution
+            observation = ErrorObservation(content=str(e))
+        except (httpx.NetworkError, AgentRuntimeDisconnectedError) as e:
+            runtime_status = RuntimeStatus.ERROR_RUNTIME_DISCONNECTED
             error_message = f'{type(e).__name__}: {str(e)}'
             self.log('error', f'Unexpected error while running action: {error_message}')
             self.log('error', f'Problematic action: {str(event)}')
-            self.set_runtime_status(runtime_status, error_message)
+            self.set_runtime_status(runtime_status, error_message, level='error')
+            return
+        except Exception as e:
+            runtime_status = RuntimeStatus.ERROR
+            error_message = f'{type(e).__name__}: {str(e)}'
+            self.log('error', f'Unexpected error while running action: {error_message}')
+            self.log('error', f'Problematic action: {str(event)}')
+            self.set_runtime_status(runtime_status, error_message, level='error')
             return
 
         observation._cause = event.id  # type: ignore[attr-defined]
@@ -381,8 +393,8 @@ class Runtime(FileEditRuntimeMixin):
                 )
             return ''
 
-        remote_repo_url = await self._get_authenticated_git_url(
-            selected_repository, git_provider_tokens
+        remote_repo_url = await self.provider_handler.get_authenticated_git_url(
+            selected_repository
         )
 
         if not remote_repo_url:
@@ -601,68 +613,6 @@ fi
 
         return loaded_microagents
 
-    async def _get_authenticated_git_url(
-        self, repo_name: str, git_provider_tokens: PROVIDER_TOKEN_TYPE | None
-    ) -> str:
-        """Get an authenticated git URL for a repository.
-
-        Args:
-            repo_path: Repository name (owner/repo)
-
-        Returns:
-            Authenticated git URL if credentials are available, otherwise regular HTTPS URL
-        """
-
-        try:
-            provider_handler = ProviderHandler(
-                git_provider_tokens or MappingProxyType({})
-            )
-            repository = await provider_handler.verify_repo_provider(repo_name)
-        except AuthenticationError:
-            raise Exception('Git provider authentication issue when getting remote URL')
-
-        provider = repository.git_provider
-        repo_name = repository.full_name
-
-        provider_domains = {
-            ProviderType.GITHUB: 'github.com',
-            ProviderType.GITLAB: 'gitlab.com',
-            ProviderType.BITBUCKET: 'bitbucket.org',
-        }
-
-        domain = provider_domains[provider]
-
-        # If git_provider_tokens is provided, use the host from the token if available
-        if git_provider_tokens and provider in git_provider_tokens:
-            domain = git_provider_tokens[provider].host or domain
-
-        # Try to use token if available, otherwise use public URL
-        if git_provider_tokens and provider in git_provider_tokens:
-            git_token = git_provider_tokens[provider].token
-            if git_token:
-                token_value = git_token.get_secret_value()
-                if provider == ProviderType.GITLAB:
-                    remote_url = (
-                        f'https://oauth2:{token_value}@{domain}/{repo_name}.git'
-                    )
-                elif provider == ProviderType.BITBUCKET:
-                    # For Bitbucket, handle username:app_password format
-                    if ':' in token_value:
-                        # App token format: username:app_password
-                        remote_url = f'https://{token_value}@{domain}/{repo_name}.git'
-                    else:
-                        # Access token format: use x-token-auth
-                        remote_url = f'https://x-token-auth:{token_value}@{domain}/{repo_name}.git'
-                else:
-                    # GitHub
-                    remote_url = f'https://{token_value}@{domain}/{repo_name}.git'
-            else:
-                remote_url = f'https://{domain}/{repo_name}.git'
-        else:
-            remote_url = f'https://{domain}/{repo_name}.git'
-
-        return remote_url
-
     def _is_gitlab_repository(self, repo_name: str) -> bool:
         """Check if a repository is hosted on GitLab.
 
@@ -760,17 +710,22 @@ fi
             # Get authenticated URL and do a shallow clone (--depth 1) for efficiency
             try:
                 remote_url = call_async_from_sync(
-                    self._get_authenticated_git_url,
+                    self.provider_handler.get_authenticated_git_url,
                     GENERAL_TIMEOUT,
                     org_openhands_repo,
-                    self.git_provider_tokens,
                 )
+            except AuthenticationError as e:
+                self.log(
+                    'debug',
+                    f'org-level microagent directory {org_openhands_repo} not found: {str(e)}',
+                )
+                raise
             except Exception as e:
                 self.log(
-                    'error',
+                    'debug',
                     f'Failed to get authenticated URL for {org_openhands_repo}: {str(e)}',
                 )
-                raise Exception(str(e))
+                raise
 
             clone_cmd = (
                 f'GIT_TERMINAL_PROMPT=0 git clone --depth 1 {remote_url} {org_repo_dir}'
@@ -826,6 +781,11 @@ fi
                     f'Clone command output: {clone_error_msg}',
                 )
 
+        except AuthenticationError as e:
+            self.log(
+                'debug',
+                f'org-level microagent directory {org_openhands_repo} not found: {str(e)}',
+            )
         except Exception as e:
             self.log(
                 'debug',
@@ -1058,6 +1018,15 @@ fi
 
         return CommandResult(content=content, exit_code=exit_code)
 
+    def _create_file_fn_git_handler(self, path: str, content: str) -> int:
+        """
+        This function is used by the GitHandler to execute shell commands.
+        """
+        obs = self.write(FileWriteAction(path=path, content=content))
+        if isinstance(obs, ErrorObservation):
+            return -1
+        return 0
+
     def get_git_changes(self, cwd: str) -> list[dict[str, str]] | None:
         self.git_handler.set_cwd(cwd)
         changes = self.git_handler.get_git_changes()
