@@ -2,19 +2,14 @@ import uvicorn
 from fastapi import FastAPI, WebSocket
 
 from openhands.core.logger import openhands_logger as logger
-from openhands.core.schema import ActionType
 from openhands.utils.shutdown_listener import should_continue
 
 app = FastAPI()
 
 
 @app.websocket('/ws')
-async def websocket_endpoint(websocket: WebSocket):
+async def websocket_endpoint(websocket: WebSocket) -> None:
     await websocket.accept()
-    # send message to mock connection
-    await websocket.send_json(
-        {'action': ActionType.INIT, 'message': 'Control loop started.'}
-    )
 
     try:
         while should_continue():
@@ -31,12 +26,12 @@ async def websocket_endpoint(websocket: WebSocket):
 
 
 @app.get('/')
-def read_root():
+def read_root() -> dict[str, str]:
     return {'message': 'This is a mock server'}
 
 
 @app.get('/api/options/models')
-def read_llm_models():
+def read_llm_models() -> list[str]:
     return [
         'gpt-4',
         'gpt-4-turbo-preview',
@@ -46,17 +41,27 @@ def read_llm_models():
 
 
 @app.get('/api/options/agents')
-def read_llm_agents():
+def read_llm_agents() -> list[str]:
     return [
         'CodeActAgent',
-        'PlannerAgent',
     ]
 
 
 @app.get('/api/list-files')
-def refresh_files():
+def refresh_files() -> list[str]:
     return ['hello_world.py']
 
 
+@app.get('/api/options/config')
+def get_config() -> dict[str, str]:
+    # return {'APP_MODE': 'oss'}
+    return {'APP_MODE': 'saas'}
+
+
+@app.get('/api/options/security-analyzers')
+def get_analyzers() -> list[str]:
+    return []
+
+
 if __name__ == '__main__':
     uvicorn.run(app, host='127.0.0.1', port=3000)
